#ifndef __DECLARATIONS_FOR_ODBC_H_
#define __DECLARATIONS_FOR_ODBC_H_


// *** Kontrollera att headerfilerna fr SQL ODBC r inkluderade annars skall dessa inkluderas ***
#ifndef __SQLTYPES
   #include "sqltypes.h"
#endif

#ifndef __SQL
   #include "sql.h"
#endif

#ifndef __SQLEXT
   #include "sqlext.h"
#endif




// = Constants ====================================================================================

#define MAX_FIELDS 20

#define ODBC_DISPLAY_ERRORINFO_IN_DEBUG    0x00000001
#define ODBC_DISPLAY_ERRORINFO_IN_RELEASE  0x00000002
#define ODBC_DISPLAY_ERRORINFO_IN_FILE     0x00000004

#define ODBC_EXEPTION_SET_STMT_OPTION  100

#define ODBC_EXEPTION_MOVE_ABSOLUTE    10
#define ODBC_EXEPTION_MOVE_RELATIVE    20
#define ODBC_EXEPTION_MOVE_BOOKMARK    30
#define ODBC_EXEPTION_MOVE_NEXT        40
#define ODBC_EXEPTION_MOVE_PREVIOUS    50
#define ODBC_EXEPTION_MOVE_LAST        60
#define ODBC_EXEPTION_MOVE_FIRST       70

#define ODBC_HSTMT_ALLOCATED           0x00000001
#define ODBC_BUFFERS_ALLOCATED         0x00000002
#define ODBC_BOOKMARKS                 0x00000004
#define ODBC_VALID_CURSOR              0x00000008
#define ODBC_CURSOR_OPEN               0x00000010
#define ODBC_NO_RECORDS                0x00000020

#define ODBC_INITIALIZE_COLUMN_NAME    0x00000001
#define ODBC_INITIALIZE_BIND           0x00000002


#define ODBC_EXEC_DIRECT               1
#define ODBC_EXEC_PREPARE              2
#define ODBC_EXEC_PREPARE_AND_EXECUTE  3






// = Structures ==================================================================================

/* ==============================================
 * Struktur fr att stta olika Villkor till
 * connections samt statement
 */
typedef struct tagODBC_OPTION
{
   UWORD  m_wOption;
   UDWORD m_dwParam;
} ODBC_OPTION, * PODBC_OPTION;

/* ==============================================
 * Structure for creating buffers an connect them
 * to columns in a result.
 */
typedef struct tagODBC_QUESTION_BUFFERINFO
{
   int m_nType;         // Flttyp
   int m_nLength;       // Lngd p textflt
   CString m_stringName;// Fltnamn
} ODBC_QUESTION_BUFFERINFO, * PODBC_QUESTION_BUFFERINFO;

/* ==============================================
 * Structure for binding parameters to a SQL-
 * question. Parameters in a SQL-qustion has
 * has questionmarks insted of att valid value.
 * The questionmark vill be replaced with 
 * assigned buffer.
 */
typedef struct tagODBC_BIND_PARAMETER
{
   HSTMT   m_hstmt;                 // Statment
   UWORD   m_wIndex;                // Index on the parameter which we are working on
   SWORD   m_wParamType;            // SQL_PARAM_INPUT, SQL_DATA_AT_EXEC ...
   SWORD   m_wCType;                // The type in C
   SWORD   m_wSQLType;              // The type in SQL
   UDWORD  m_dwPrecision;           // Number of valid signs in column
   SWORD   m_wScale;                // 
   PTR     m_pBuffer;               // Pointer to buffer where value are stored or inserted
   SDWORD  m_dwMaxBytes;            // Max bytes that are transfered
   SDWORD* m_pdwBytesRetrieved;     // Bytes count that was tranfered. 
} ODBC_BIND_PARAMETER, * PODBC_BIND_PARAMETER;

/* ==============================================
 * This structure can be used for filling columns
 * with data at execution time. The type of 
 * column this mostly aplies to is binary 
 * columns.
 */
typedef struct tagODBC_DATA_AT_EXECUTION
{
   short    m_sParameterIndex;       // Index for the parameter that data is transfered for.
   UINT     m_uBufferLength;         // Number of bytes that should be transmitted
   void*    m_pBuffer;               // Pointer to buffer where data could be found
} ODBC_DATA_AT_EXECUTION, * PODBC_DATA_AT_EXECUTION;

/* ==============================================
 * Structure for retriving data from a column.
 */
typedef struct tagODBC_GET_DATA
{
   UWORD   wColumn;     // Index for column which data shold be fetched from
   SWORD   wCType;      // The datatype in C
   PTR     pBuffer;     // Pointer to buffer that data should be transfered to.
   SDWORD  dwMaxBytes;  // Max size for data that shold be transfered
   SDWORD  dwByteRead;  // Data that has been transfered
} ODBC_GET_DATA, * PODBC_GET_DATA;



// = Macro ========================================================================================

#ifdef _DEBUG
   #define CHECK(text) \
      if( CheckODBCError() == FALSE ) throw text;
#else
   #define CHECK(text) \
      if( CheckODBCError() == FALSE ) throw (char*)NULL;   
#endif

#define CHECK_FOR_ERROR()  CheckODBCError();
#define CHECK_AND_BREAK() \
   if( CheckODBCError() == FALSE ) return FALSE;
#define CHECK_AND_THROW() \
   if( CheckODBCError() == FALSE ) throw 0;   

#define ODBC_BIND_PARAMETER( par, hstmt, index, type, c_type, sql_type, prec, buff, max, read ) \
   par.m_hStmt = hstmt;    par.m_wIndex = index;         par.m_wParamType = type;   \
   par.m_wCType = c_type;  par.m_wSQLType = sql_type;    par.m_dwPrecision = prec;  \
   par.m_wScale = 0;       par.m_pBuffer = buff;         par.m_dwMaxBytes = max;    \
   par.m_pdwBytesRetrieved = read;

#define ODBC_BIND_PARAMETER_BINARY( par, index, type, byte_count, buffer, read ) \
   par.m_hstmt = SQL_NULL_HSTMT;par.m_wIndex = index;               par.m_wParamType = type;   \
   par.m_wCType = SQL_C_BINARY; par.m_wSQLType = SQL_LONGVARBINARY; par.m_dwPrecision = byte_count;  \
   par.m_wScale = 0;            par.m_pBuffer = buffer;             par.m_dwMaxBytes = byte_count + 1;   \
   par.m_pdwBytesRetrieved = read;     

#define ODBC_DATA_AT_EXECUTION( data, index, length, buffer ) \
   data.m_sParameterIndex = index; data.m_uBufferLength = length; data.m_pBuffer = buffer; 

#define ODBC_GET_DATA( data, column, ctype, buffer, max ) \
   data.wColumn = column; data.wCType = ctype; data.pBuffer = buffer; data.dwMaxBytes = max;


#ifndef __ODBCDATABASE_H_
   #include "ODBCDatabase.h"
#endif

#ifndef __ODBCRECORDSET_H_
   #include "ODBCRecordSet.h"
#endif

#endif