/*
   FILNAMN:    odbcdatabase.cpp

   SYFTE:      Hanterar en databas med ODBC-API

   SKAPAD:     951004 Per Ghosh
   MILJ:      Visual C++


INFO:


NDRINGSLOG:
Sign.. Datum... Kommentarer........................................................................

*/

#include "stdafx.h"

#include "DECLARATIONS_FOR_ODBC.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

#ifdef _DEBUG
static char* module_ppErrMsg[] =
{
   {"Fail to allocate enviroment handle"},
   {"Fail to allocate connection handle"},
   {"Fail to connect"},
   NULL
};

#define ERR_ODBC_HENV      0
#define ERR_ODBC_HDBC      1
#define ERR_ODBC_CONNECT   2
#endif

IMPLEMENT_DYNAMIC(CODBCDatabase, CObject)

/************************************************
   *   Konstruktor/Destruktor 
   *
 * * *
  ***
   *
*/

CODBCDatabase::CODBCDatabase()   :
   m_henv(SQL_NULL_HENV), m_hdbc(SQL_NULL_HDBC), m_hstmt(SQL_NULL_HSTMT), 
   m_retcode(0), m_pszSourceName(NULL), m_pszUserId(NULL), m_pszPassword(NULL)
{
   m_bConnected = FALSE;
}

CODBCDatabase::~CODBCDatabase()
{
   if( m_bConnected )
   {
      Disconnect();
   }
}

/************************************************
   *   Virtuella funktioner
   *
 * * *
  ***
   *
*/

/************************************************
   *   Funktioner                                          SQLPutData
   *
 * * *
  ***
   *
*/

/*-------------------------------------------------------------------------------------------------
 Namn:   Connect()
 Typ:    funktion
 Syfte:  koppla upp oss mot en databas
*/
BOOL CODBCDatabase::Connect( LPCSTR pszSourceName, 
                             LPCSTR pszUserId /*=NULL*/, 
                             LPCSTR pszPassword /*=NULL*/ )
{
   ASSERT( m_henv == SQL_NULL_HENV );

   BOOL  bReturn = TRUE;
   SWORD wLengthUID = 0;
   SWORD wLengthPSW = 0;

   try
   {
      // *** allocate handle for m_henv (allocate this first) ***
      if( m_henv == SQL_NULL_HENV )
      {
         m_retcode = ::SQLAllocEnv( &m_henv );
         CHECK(module_ppErrMsg[ERR_ODBC_HENV])
      }

      // *** allocate handle for m_hdbc ***
      if( m_henv != SQL_NULL_HENV )
      {
         m_retcode = ::SQLAllocConnect( m_henv, &m_hdbc );
         CHECK(module_ppErrMsg[ERR_ODBC_HDBC])
      }

      m_pszSourceName = new char[strlen(pszSourceName) + 1];
      strcpy( m_pszSourceName, pszSourceName );

      if( pszUserId != NULL )       // Is there a user-id
      {
         m_pszUserId = new char[strlen(pszUserId) + 1];
         strcpy( m_pszUserId, pszUserId );
         wLengthUID = SQL_NTS;
      }

      if( pszPassword != NULL )     // Is there a password
      {
         m_pszPassword = new char[strlen(pszPassword) + 1];
         strcpy( m_pszPassword, pszPassword );
         wLengthPSW = SQL_NTS;
      }

      // Connect to datasource
      m_retcode = ::SQLConnect( m_hdbc, (PUCHAR)pszSourceName, SQL_NTS, 
                                        (PUCHAR)pszUserId, wLengthUID, 
                                        (PUCHAR)pszPassword, wLengthPSW );
      CHECK(module_ppErrMsg[ERR_ODBC_CONNECT])
      m_bConnected = TRUE;
      TRACE1("Connected to datasource\"%s\"\n", m_pszSourceName );
      
   }
   catch( char* pszText )
   {
      TRACE1("%s\n",pszText);
      bReturn = FALSE;
   }
   catch(...)
   {
      bReturn = FALSE;
   }

   return bReturn;
}

/*-------------------------------------------------------------------------------------------------
 Namn:   Disconnect()
 Typ:    funktion
 Syfte:  Koppla loss oss frn en databas
*/
void CODBCDatabase::Disconnect()
{
   if( m_pszSourceName ) 
   {
      delete m_pszSourceName;
      m_pszSourceName = NULL;
   }

   if( m_pszUserId )
   {
      delete m_pszUserId;
      m_pszUserId = NULL;
   }

   if( m_pszPassword ) 
   {
      delete m_pszPassword;
      m_pszPassword = NULL;
   }

   if( m_hdbc )
   {
      if( m_bConnected )
      {
         m_retcode = ::SQLDisconnect( m_hdbc );
         CHECK_FOR_ERROR()
         m_bConnected = FALSE;
      }
      m_retcode = ::SQLFreeConnect( m_hdbc );
      CHECK_FOR_ERROR()
      m_hdbc = SQL_NULL_HDBC;
   }

   if( m_henv )
   {
      m_retcode = ::SQLFreeEnv( m_henv );
      CHECK_FOR_ERROR()
      m_henv = SQL_NULL_HENV;
   }
}

/*-------------------------------------------------------------------------------------------------
Name:	   RemoveRecordSet()
Type:    Function
Purpose: Remove pointer to a CODBCRecordSet from list

Param:   pODBCRecordSet = pointer to CODBCRecordSet wich are going to be removed
*/
void CODBCDatabase::RemoveRecordSet( CODBCRecordSet* pODBCRecordSet )
{
   POSITION pos;
   POSITION posRemove;
   CODBCRecordSet* pODBCRS;

   pos = m_listToODBCRecordSet.GetHeadPosition();
   while( pos )
   {
      posRemove = pos;
      pODBCRS = (CODBCRecordSet*)m_listToODBCRecordSet.GetNext( pos );
      if( pODBCRS == pODBCRecordSet )
      {
         m_listToODBCRecordSet.RemoveAt( posRemove );
         return;
      }
   }

   TRACE0("Did not find pointer to CODBCRecordSet\n");
}

/*-------------------------------------------------------------------------------------------------
Name:	 
Type:    
Purpose: 

Param:

Return:
*/
BOOL CODBCDatabase::ExecuteDirect( LPCSTR pszSQL )
{
   ASSERT( m_hdbc );

   BOOL bReturn;
   char* pszTempSQL;

   pszSQL = new char[strlen(pszSQL) + 1];
   strcpy( pszTempSQL, pszSQL );

   bReturn = ExecuteDirect( pszTempSQL );

   delete pszTempSQL;

   return bReturn;
}

//-------------------------------------------------------------------------------------------------
BOOL CODBCDatabase::ExecuteDirect( char* pszSQL )
{
   ASSERT( m_hdbc );

   BOOL bReturn = TRUE;

   m_retcode = ::SQLAllocStmt( m_hdbc, &m_hstmt );
   CHECK_AND_BREAK()
   m_retcode = ::SQLExecDirect( m_hstmt, (PUCHAR)pszSQL, SQL_NTS );
   bReturn = CHECK_FOR_ERROR()
   m_retcode = ::SQLFreeStmt( m_hstmt, SQL_DROP );
   CHECK_FOR_ERROR()

   m_hstmt = SQL_NULL_HSTMT;

   return bReturn;
}

//-------------------------------------------------------------------------------------------------
BOOL CODBCDatabase::Execute( LPCSTR pszSQL )
{
   ASSERT( m_hdbc );

   BOOL bReturn = TRUE;

   m_pszTempSQL = new char[strlen(pszSQL)+1];
   strcpy( m_pszTempSQL, pszSQL );

   bReturn = Execute();

   return bReturn;
}

//-------------------------------------------------------------------------------------------------
BOOL CODBCDatabase::Execute( char* pszSQL )
{
   ASSERT( m_hdbc );

   BOOL bReturn;

   m_pszTempSQL = pszSQL;
   bReturn = Execute();

   return bReturn;
}


  

/************************************************
   *   Debug
   *
 * * *
  ***
   *
*/

#ifdef _DEBUG
void CODBCDatabase::AssertValid() const
{
	CObject::AssertValid();
}

void CODBCDatabase::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
}
#endif //_DEBUG

/*-------------------------------------------------------------------------------------------------
Name:	 
Type:    
Purpose: 

Return:
*/
BOOL CODBCDatabase::Execute()
{
   ASSERT( m_hdbc );

   BOOL bReturn = TRUE;

   try
   {
      m_retcode = ::SQLAllocStmt( m_hdbc, &m_hstmt );
      CHECK_AND_THROW()
      m_retcode = ::SQLPrepare( m_hstmt, (PUCHAR)m_pszTempSQL, SQL_NTS );
      CHECK_AND_THROW()   
      m_retcode = ::SQLExecute( m_hstmt );
      CHECK_AND_THROW()   
      m_retcode = ::SQLFreeStmt( m_hstmt, SQL_DROP );
      bReturn = CHECK_FOR_ERROR()
   }
   catch(...)
   {
      bReturn = FALSE;

      if( m_hstmt )
      {
         ::SQLFreeStmt( m_hstmt, SQL_DROP );
      }
   }

   m_hstmt = SQL_NULL_HSTMT;
   return bReturn;
}



/*-------------------------------------------------------------------------------------------------
 Namn:   DisplayODBCError()
 Typ:    Funktion
 Syfte:  Show information about error that are stored in driver

 Param:  hstmt = show information about error to statement, is statement SQL_NULL_HSTMT  than
         display all error's in driver
*/
void CODBCDatabase::DisplayODBCError( HSTMT hstmt /*=SQL_NULL_HSTMT*/ )
{
   UCHAR   pszSqlState[6];    // Pekare till status fr felmedelande-kod (ett nummer)
   SDWORD  dwNativeError;     // Specifik fel-kod fr den drivaren som r kopplad
   PUCHAR  pszErrorMsg;       // Pekare till felmeddelande
   SWORD   wMaxLength;        // Max lngd som felmedelandet fr lov att ha.
   SWORD   wErrorMsgLength;   // Lngden p felmedelandet

   pszErrorMsg = new UCHAR[SQL_MAX_MESSAGE_LENGTH];
   wMaxLength  = SQL_MAX_MESSAGE_LENGTH;

   // *** Kolla om det finns ngra fel som r kopplade till statment ***
   if( hstmt )
   {
      while( ::SQLError( SQL_NULL_HENV, 
                         SQL_NULL_HDBC, 
                         hstmt, 
                         pszSqlState,
                         &dwNativeError,
                         pszErrorMsg,
                         wMaxLength,
                         &wErrorMsgLength ) == SQL_SUCCESS )
      {
         // Vi har hittat ett fel
         PrintErrorInfo( pszSqlState, dwNativeError, pszErrorMsg ); 
      }
      return;
   }

   if( m_hstmt )
   {
      while( ::SQLError( SQL_NULL_HENV, 
                         SQL_NULL_HDBC, 
                         m_hstmt, 
                         pszSqlState,
                         &dwNativeError,
                         pszErrorMsg,
                         wMaxLength,
                         &wErrorMsgLength ) == SQL_SUCCESS )
      {
         // Vi har hittat ett fel
         PrintErrorInfo( pszSqlState, dwNativeError, pszErrorMsg ); 
      }
   }

   // *** Kolla om det finns ngra fel kopplade till konnection ***
   if( m_hdbc )
   {
      while( ::SQLError( SQL_NULL_HENV, 
                         m_hdbc, 
                         SQL_NULL_HSTMT, 
                         pszSqlState,
                         &dwNativeError,
                         pszErrorMsg,
                         wMaxLength,
                         &wErrorMsgLength ) == SQL_SUCCESS )
      {
         // Vi har hittat ett fel
         PrintErrorInfo( pszSqlState, dwNativeError, pszErrorMsg ); 
      }
   }

   // Kolla om det finns ngra fel kopplade till enviroment ***
   if( m_henv )
   {
      while( ::SQLError( m_henv, 
                         SQL_NULL_HDBC, 
                         SQL_NULL_HSTMT, 
                         pszSqlState,
                         &dwNativeError,
                         pszErrorMsg,
                         wMaxLength,
                         &wErrorMsgLength ) == SQL_SUCCESS )
      {
         // Vi har hittat ett fel
         PrintErrorInfo( pszSqlState, dwNativeError, pszErrorMsg ); 
      }
   }

   delete pszErrorMsg;
}

/*-------------------------------------------------------------------------------------------------
Name:	 
Type:    
Purpose: 

Param:

Return:
*/
void CODBCDatabase::PrintErrorInfo( PUCHAR pszState, SDWORD dwNativeError, PUCHAR pszErrorMsg )
{
   CString stringError;
   char    pszNativeError[20];
   sprintf( pszNativeError, "NATIVE = %d\n", dwNativeError );

   stringError += "STATE = ";
   stringError += pszState;
   stringError += '\n';
   stringError += pszNativeError;
   stringError += pszErrorMsg;

   if( m_uShowErrorInfo & ODBC_DISPLAY_ERRORINFO_IN_DEBUG )
   {
      TRACE0(stringError);
   }

   if( m_uShowErrorInfo & ODBC_DISPLAY_ERRORINFO_IN_RELEASE )
   {
      ::AfxMessageBox(stringError);
   }
}



