/*
   FILNAMN:    odbcdatabase.h

   SYFTE:      Hanterar en databas med ODBC-API

   SKAPAD:     951004 Per Ghosh
   MILJ:      Visual C++


INFO:
                                           

NDRINGSLOG:
Sign.. Datum... Kommentarer........................................................................

*/

#ifndef __ODBCDATABASE_H_
#define __ODBCDATABASE_H_

// *** Kontrollera att headerfilerna fr SQL ODBC r inkluderade annars skall dessa inkluderas ***

class CODBCDatabase : public CObject
{
   friend class CODBCRecordSet;
	DECLARE_DYNAMIC(CODBCDatabase)

// constructor/destructor
public:
	CODBCDatabase();           
	virtual ~CODBCDatabase();

// Attributes
private:
   HENV     m_henv;           // Enviroment handle 
   HDBC     m_hdbc;           // SQLAllocconnect

   BOOL     m_bConnected;     // Markerar om vi kopplat upp oss mot en databas

   char*    m_pszSourceName;  // Registrerat databasnamn i ODBC
   char*    m_pszUserId;      // Anvndar identifikation
   char*    m_pszPassword;    // Password

   char*    m_pszTempSQL;     // SQL-strng som krs mot databasen
   HSTMT    m_hstmt;          // Handtag till statement

   RETCODE  m_retcode;        // Variabel fr att hantera felinformation

   UINT     m_uShowErrorInfo; // Skall felinformation visas

   CPtrList m_listToODBCRecordSet;

// Get/Set
public:
   HENV  GetHENV() { return m_henv; }
   HDBC  GetHDBC() { return m_hdbc; }
   char* GetSourceName() { return m_pszSourceName; }
   char* GetUserId() { return m_pszUserId; }
   char* GetPassword() { return m_pszPassword; }

// Operations
public:
   // Kr igng en ODBC-drivare samt koppla upp oss mot en tabell
   BOOL Connect( LPCSTR pszSourceName, LPCSTR pszUserId = NULL, LPCSTR pszPassword = NULL );
   // Koppla bort oss frn databasen och slpp ODBC-drivarn
   void Disconnect();
   inline void AddRecordSet( CODBCRecordSet* pODBCRecordSet );
	void RemoveRecordSet( CODBCRecordSet* pODBCRecordSet );

   // Kr en frga mot databasen
   BOOL ExecuteDirect( LPCSTR pszSQL );
   BOOL ExecuteDirect( char* pszSQL );
   BOOL Execute( LPCSTR pszSQL );
   BOOL Execute( char* pszSQL );
   RETCODE Execute( HSTMT hstmt ) { return ::SQLExecute( m_hstmt ); }
   // Visa felinformation
   void DisplayODBCError( HSTMT hstmt = SQL_NULL_HSTMT );
   
   // Kontrollerar felkod
   inline BOOL CheckODBCError();
   inline BOOL CheckODBCError( RETCODE retcode );


// Implementation
private:
	void PrintErrorInfo( PUCHAR pszState, SDWORD dwNativeError, PUCHAR pszErrorMsg );
   BOOL Execute();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif


};
                                   
inline void CODBCDatabase::AddRecordSet( CODBCRecordSet* pODBCRecordSet )
{
   m_listToODBCRecordSet.AddTail( pODBCRecordSet );
}

inline BOOL CODBCDatabase::CheckODBCError()
{
   if( m_retcode == SQL_SUCCESS )
   {
      return TRUE;
   }
   else
   {
      DisplayODBCError();
   }
   return FALSE;
}

/*-----------------------------------------------
 Namn:   CheckODBCError()
 Typ:    inline funktion
 Syfte:  Kontrollera eventuell felkod
*/
inline BOOL CODBCDatabase::CheckODBCError( RETCODE retcode )
{
   if( retcode == SQL_SUCCESS )
   {
      return TRUE;
   }
   else
   {
      DisplayODBCError();
   }

   return FALSE;
}

#endif