/*
   FILNAMN:    ODBCRecordSet.cpp

   DELSYSTEM:  

   SYFTE:      Kunna kontrollera statements via ODBC

   SKAPAD:     951004 Per Ghosh
   MILJ:      Visual C++


INFO:


*/

#include "stdafx.h"

#include "DECLARATIONS_FOR_ODBC.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CODBCRecordSet

IMPLEMENT_DYNAMIC(CODBCRecordSet, CObject)

static SDWORD module_pdwBufferSize[] =
{
   0,                                 
   0,   // SQL_CHAR    1
   0,   // SQL_NUMERIC 2
   0,   // SQL_DECIMAL 3
   4,   // SQL_INTEGER 4
   2,   // SQL_SMALLINT5
   8,   // SQL_FLOAT   6
   8,   // SQL_REAL    7
   8    // SQL_DOUBLE  8
};

static DWORD dwDummy;



/************************************************
   *   Konstruktor/Destruktor 
   *
 * * *
  ***
   *
*/

CODBCRecordSet::CODBCRecordSet( CODBCDatabase* pODBCDatabase ) :
   m_hstmt(SQL_NULL_HSTMT), m_uFieldCount(0), m_wRowStatus(0), m_uFlags(0), m_retcode(0)
{
   ASSERT(pODBCDatabase->IsKindOf( RUNTIME_CLASS(CODBCDatabase) ) );
   
   m_pODBCDatabase = pODBCDatabase;
   m_RecordPos = Closed;
}

CODBCRecordSet::~CODBCRecordSet()
{
   if( m_uFlags & ODBC_CURSOR_OPEN )  // Check if the cursor is valid
   {
      // TRACE0("Warning cursor is not closed before deleting objekt\n");
      Close(); // Close statement
   }

   while( m_uFieldCount )
   {
      m_uFieldCount--;

      if( m_ptrarrayFieldValue[m_uFieldCount] )
      {
         delete m_ptrarrayFieldValue[m_uFieldCount];
      }
   }
#ifdef _DEBUG
    if( m_ptrarrayFieldValue.GetSize() )
       TRACE0("You have more pointer's in \"m_ptrarrayFieldValue\"\n" );
#endif
}

/************************************************
   *   Funktioner 
   *
 * * *
  ***
   *
*/

/*-------------------------------------------------------------------------------------------------
Name:	   AllocStmt()
Type:    Function
Purpose: Allocate a statement

Return:
*/
BOOL CODBCRecordSet::AllocStmt()
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt == SQL_NULL_HSTMT );

   // Allokera memory for a statement
   m_retcode = ::SQLAllocStmt( m_pODBCDatabase->m_hdbc, &m_hstmt );
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError();
      return FALSE;
   }

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Open()
 Type:   Function
 Purpose:Execute a question and attach a cursor.

 Param:  pszSQL = pointer to string that holds the question.
*/
BOOL CODBCRecordSet::Open( LPCSTR pszSQL )
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt == SQL_NULL_HSTMT );
   ASSERT( m_uFlags & ODBC_BUFFERS_ALLOCATED );

   BOOL bReturn = TRUE;

   // Allokera minne fr ett statement
   m_retcode = ::SQLAllocStmt( m_pODBCDatabase->m_hdbc, &m_hstmt );
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError();
      return FALSE;
   }

   // Set cursor-type (STATIC, DYNAMIC, KEYSET ...)
   if( SetStmtOption( SQL_CURSOR_TYPE, SQL_CURSOR_STATIC) == FALSE ) return FALSE;

   // Prepare question
   if( Prepare( pszSQL ) == FALSE ) return FALSE;
   
   // *** Bind columns ***
   if( m_uFlags & ODBC_BUFFERS_ALLOCATED );
   {
      if( BindColumns() == FALSE ) return FALSE;
   }

   // Execute the question
   if( Execute() == FALSE ) return FALSE;

   MoveFirst();

   if( m_RecordPos != IsOnValidRecord )
   {
      m_uFlags |= ODBC_NO_RECORDS;
   }

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Open()
 Type:   Function
 Purpose:Execute a question and attach a cursor.

 Param:  pszSQL = pekare till den strng som innehller SQL-frgan
         pOptions = Antal villkor som skall sttas fr cursorn
         
*/
BOOL CODBCRecordSet::Open( LPCSTR pszSQL,  WORD wOptionCount, PODBC_OPTION pOptions )
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt == SQL_NULL_HSTMT );
   ASSERT( m_uFlags & ODBC_BUFFERS_ALLOCATED );

   BOOL bReturn = TRUE;


   // Allokera minne fr ett statement
   m_retcode = ::SQLAllocStmt( m_pODBCDatabase->m_hdbc, &m_hstmt );
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError();
      return FALSE;
   }

   // Set cursor-type (STATIC, DYNAMIC, KEYSET ...)
   if( SetStmtOption( wOptionCount, pOptions ) == FALSE ) return FALSE;

   // Prepare question
   if( Prepare( pszSQL ) == FALSE ) return FALSE;
   
   // *** Bind columns ***
   if( m_uFlags & ODBC_BUFFERS_ALLOCATED );
   {
      if( BindColumns() == FALSE ) return FALSE;
   }

   // Execute the question
   if( Execute() == FALSE ) return FALSE;

   MoveFirst();

   if( m_RecordPos != IsOnValidRecord )
   {
      m_uFlags |= ODBC_NO_RECORDS;
   }

   return bReturn;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Open()
 Type:   Function
 Purpose:Execute a question and attach a cursor.

 Param:  pszSQL = Pointer to SQL-question
         uCursorType = what kind of cursor is going to be attached
         bBind = if columns in the result should be attached to buffers
         bInitialize = if buffers should be made for the result

 Return: BOOL, TRUE if everything went ok, otherwise FALSE
*/
BOOL CODBCRecordSet::Open( LPCSTR pszSQL, 
                           UINT uCursorType, 
                           BOOL bBind /*=TRUE*/, 
                           BOOL bInitialize /*=FALSE*/ )
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt == SQL_NULL_HSTMT );
   ASSERT( uCursorType <= SQL_CURSOR_STATIC );
   ASSERT( ! (bInitialize & (m_uFlags & ODBC_BUFFERS_ALLOCATED) ) );
   // Have to allocate buffers if values should be binded
   ASSERT( !( bBind && (bInitialize == FALSE) && (!(m_uFlags & ODBC_BUFFERS_ALLOCATED)) ) );
   BOOL bReturn = TRUE;


   // Allokera minne fr ett statement
   m_retcode = ::SQLAllocStmt( m_pODBCDatabase->m_hdbc, &m_hstmt );
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError();
      return FALSE;
   }

   // Set statement-option before preparing it
   if( SetStmtOption( SQL_CURSOR_TYPE, uCursorType) == FALSE ) return FALSE;

   // Prepare question
   if( Prepare( pszSQL ) == FALSE ) return FALSE;

   // Execute the question
   if( Execute() == FALSE ) return FALSE;

   // Should we make buffers by investigating the columns in result
   if( bInitialize == TRUE )
   {
      if( Initialize() == FALSE ) return FALSE;
   }

   // Should the columns be binded to created buffers
   if( bBind == TRUE )
   {
      if( BindColumns() == FALSE ) return FALSE;
   }

   if( uCursorType != SQL_CURSOR_FORWARD_ONLY )
   {
      MoveFirst();
   }
   else
   {
      TRACE0("Do you realy want this (SQL_CURSOR_FORWARD_ONLY) kind of cursor\n");
      ::SQLFetch( m_hstmt );
   }

   if( m_RecordPos != IsOnValidRecord )
   {
      m_uFlags |= ODBC_NO_RECORDS;
   }

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:    Prepare()
Type:    Function
Purpose: Prepare a question

Param:   pszSQL = pointer to SQL-question which is going to be prepared
*/
BOOL CODBCRecordSet::Prepare( LPCSTR pszSQL )
{
   ASSERT( m_hstmt != SQL_NULL_HSTMT );

   m_retcode = ::SQLPrepare(m_hstmt,(UCHAR*)pszSQL, SQL_NTS);
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->m_hstmt = m_hstmt;
      m_pODBCDatabase->DisplayODBCError();
      return FALSE;
   }
   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:    SetStmtOption()
Type:    Function
Purpose: Med denna Function kan vi stta olika status fr det statement som  bearbetas

Param:   wOption = Vilken typ av villkor som skall sttas
         uParam = bestmmer villkoret

Return:  BOOL, TRUE if everything went ok, otherwise FALSE
*/
BOOL CODBCRecordSet::SetStmtOption( WORD wOption, UINT uParam )
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );

   m_retcode = ::SQLSetStmtOption( m_hstmt, (UWORD)wOption, (UDWORD)uParam );
   
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }
   
   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   SetStmtOption()
 Type:   Function
 Purpose:Stt ett flerta vilkor fr en cursor

 Param:  uOptionCount = antal villkor
         pOptions = pekare till en vektor med tagODBC_OPTION-strukturer
*/
BOOL CODBCRecordSet::SetStmtOption( UINT uOptionCount, PODBC_OPTION pOptions )
{
   while( uOptionCount != 0 )
   {
      uOptionCount--;

      if( SetStmtOption( pOptions[uOptionCount].m_wOption, 
                         pOptions[uOptionCount].m_dwParam ) == FALSE )
      {
         return FALSE;
      }

   }// endwhile

   return TRUE;

}

/*-------------------------------------------------------------------------------------------------
 Name:   Initialize()
 Type:   Function
 Purpose:Create buffer's to store values for different columns in the result.
*/
BOOL CODBCRecordSet::Initialize( const CDWordArray& dwarray )
{
   ASSERT_VALID( this );
   ASSERT( !(m_uFlags & ODBC_BUFFERS_ALLOCATED) );

   BOOL bReturn = TRUE;
   void* pVoid;
   int   iCounter = 0;

   try
   {
      while( iCounter < dwarray.GetSize() )
      {
         m_dwarrayColumnType.Add( dwarray[iCounter] );
         ASSERT( LOWORD( dwarray[iCounter] ) <= SQL_TYPE_MAX );

         switch( LOWORD( dwarray[iCounter] ) )
         {
         case SQL_CHAR :
         case SQL_VARCHAR :
            pVoid = new char[HIWORD(dwarray[iCounter])];
            break;
         case SQL_INTEGER :
            pVoid = new long;
            break;
         case SQL_SMALLINT :
            pVoid = new short;
            break;
         case SQL_NUMERIC :
            pVoid = new double;
            break;
         case SQL_DOUBLE :
         case SQL_DECIMAL :
            pVoid = new char[HIWORD(dwarray[iCounter])];
            break;
         default:
            pVoid = NULL;
         }// endswitch

         m_uFieldCount++;
         m_ptrarrayFieldValue.Add( pVoid );
         iCounter++;
   
      }// endwhile
      
      m_uFlags |= ODBC_BUFFERS_ALLOCATED;
   }
   catch(...)
   {
      TRACE0("CODBCRecordSet::Initialize()\nERROR: Fel nr vid skapning av buffertar till de olika kolumnerna.\n");
      bReturn = FALSE;
   }

   return bReturn;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Initialize()
 Type:   Function
 Purpose:Create buffer's to store values for different columns in the result.
 
 Param:  pdwInfo = pointer to an array of DWORD's with informaiton how the buffers should be made 
                   for columns in the result.
 Return: BOOL, TRUE if everything went ok, otherwise FALSE
*/
BOOL CODBCRecordSet::Initialize( const DWORD* pdwInfo )
{
   ASSERT_VALID( this );
   ASSERT( !(m_uFlags & ODBC_BUFFERS_ALLOCATED) );

   BOOL  bReturn = TRUE;
   void* pVoid;

   try
   {
      while( pdwInfo[m_uFieldCount] )
      {
         ASSERT( m_uFieldCount < 50 );
         ASSERT( LOWORD( pdwInfo[m_uFieldCount] ) <= SQL_TYPE_MAX );

         m_dwarrayColumnType.Add( pdwInfo[m_uFieldCount] );
         switch( LOWORD(pdwInfo[m_uFieldCount] ) )
         {
         case SQL_CHAR :
         case SQL_VARCHAR :
            pVoid = new char[HIWORD(pdwInfo[m_uFieldCount])];
            break;
         case SQL_INTEGER :
            pVoid = new long;
            break;
         case SQL_SMALLINT :
            pVoid = new short;
            break;
         case SQL_DOUBLE :
            pVoid = new double;
            break;
         case SQL_NUMERIC :
         case SQL_DECIMAL :
            pVoid = new char[HIWORD(pdwInfo[m_uFieldCount])];
            break;
         default:
            pVoid = NULL;
         }// endswitch
         m_ptrarrayFieldValue.Add( pVoid );
         m_uFieldCount++;
      }// endwhile

      m_uFlags |= ODBC_BUFFERS_ALLOCATED;
   }
   catch(...)
   {
      TRACE0("CODBCRecordSet::Initialize()\nERROR: Fel nr vid skapning av buffertar till de olika kolumnerna.\n");
      bReturn = FALSE;
   }

   return bReturn;
} 

/*-------------------------------------------------------------------------------------------------
Name:    Initialize()
Type:    Function
Purpose: Create buffers for a result

Return:  BOOL, TRUE if everything went ok, otherwise FALSE
*/
BOOL CODBCRecordSet::Initialize()
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );
    
   SWORD  wColumnCount;     // Antal kolumner i resultatet
   UWORD  wColumnIndex = 0; // Index p den kolumn som vi skall bearbeta
   SWORD  wColumnType;      // Vilken typ av kolumn
   UDWORD dwPrecision;      // Vilken precision (signifikanta siffror, lngd p textflt)
   SWORD  wScale;           // Decimaler fr aktuell kolumn
   SWORD  wNullable;        // Om kolumnen kan nollas (OBS! inte samma som 0)

   void*  pVoid;            // Pekare fr att vi skall kunna skapa upp buffertar

   m_retcode = ::SQLNumResultCols( m_hstmt, &wColumnCount );
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }

   while( m_uFieldCount < (UINT)wColumnCount )
   {
      m_uFieldCount++;
      wColumnIndex = m_uFieldCount;
      m_retcode = ::SQLDescribeCol( m_hstmt, 
                                    wColumnIndex,  // Index for column that we want inforamtion about
                                    NULL, 0, NULL, // We don't want columnname
                                    &wColumnType,  // columntype
                                    &dwPrecision,  // Nogranhet eller lngd p kolumn           
                                    &wScale,
                                    &wNullable );
      if( m_retcode != SQL_SUCCESS )
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         return FALSE;
      }

      switch( wColumnType ) 
      {
      case SQL_CHAR :
      case SQL_VARCHAR :
         dwPrecision++;
         pVoid = new char[dwPrecision];
         wColumnType = SQL_CHAR;
         break;
      case SQL_INTEGER :
         pVoid = new long;
         break;
      case SQL_SMALLINT :
         pVoid = new short;
         break;
      case SQL_DOUBLE  :
         pVoid = new double;
         break;
      case SQL_NUMERIC :
      case SQL_DECIMAL :
         dwPrecision += 3;
         pVoid = new char[dwPrecision];
         break;
      default:
         pVoid = NULL;
      }// endswitch
      m_ptrarrayFieldValue.Add( pVoid );  
      m_dwarrayColumnType.Add( MAKELONG( wColumnType, (WORD)dwPrecision ) );
   }

   m_uFlags |= ODBC_BUFFERS_ALLOCATED;

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Execute()                           
 Type:   Function
 Purpose:Kr en exekevering av ett preparerat statment

 Param:  hstmt = det statment som skall exekveras

 Return: BOOL, TRUE if everything went ok, otherwise FALSE
*/
BOOL CODBCRecordSet::Execute( HSTMT hstmt /*=SQL_NULL_HSTMT*/ )
{
   ASSERT( this );

   if( hstmt == SQL_NULL_HSTMT )
   {
      hstmt = m_hstmt;
      m_uFlags |= ODBC_CURSOR_OPEN;
   }

   m_retcode = ::SQLExecute(hstmt);
   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode == SQL_ERROR || m_retcode == SQL_INVALID_HANDLE )
      {
         m_pODBCDatabase->DisplayODBCError(m_hstmt);
         m_uFlags &= ~ODBC_CURSOR_OPEN;
         return FALSE;
      }
   }

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:	   Execute()
Type:    Funktion
Purpose: Executes a question 

Param:   pszSQL  = SQL-string that are going to be executeted
         pHstmt  = pointer to statement. if this i SQL_NULL_HSTMT then the statement will be droped
         uOption = Whar we are going to with the SQL-string .

Return:  BOOL, TRUE if everything went ok, otherwise FALSE
*/
BOOL CODBCRecordSet::Execute( LPCSTR pszSQL, 
                              HSTMT* pHstmt /*=SQL_NULL_HSTMT*/, 
                              UINT uOption /*=ODBC_EXEC_DIRECT*/ )
{
   ASSERT( this );

   HSTMT hstmt;
   
   // *** Allocate statement ***
   m_retcode = ::SQLAllocStmt( m_pODBCDatabase->m_hdbc, &hstmt );
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError();
      return FALSE;
   }

   // *** What operation has been requested ***
   switch( uOption )
   {
   case ODBC_EXEC_DIRECT :
      m_retcode = ::SQLExecDirect( hstmt, (UCHAR*)pszSQL, SQL_NTS );
      break;

   case ODBC_EXEC_PREPARE :
      ASSERT(pHstmt != SQL_NULL_HSTMT);
      m_retcode = ::SQLPrepare( hstmt, (UCHAR*)pszSQL, SQL_NTS );
      break;

   case ODBC_EXEC_PREPARE_AND_EXECUTE :
      ASSERT(pHstmt != SQL_NULL_HSTMT);
      m_retcode = ::SQLPrepare( hstmt, (UCHAR*)pszSQL, SQL_NTS );
      if( m_retcode != SQL_SUCCESS )
      {
         m_pODBCDatabase->DisplayODBCError( hstmt );
         ::SQLFreeStmt( hstmt, SQL_DROP );
         return FALSE;
      }
      m_retcode = ::SQLExecute( hstmt );
      break;

   default:
      ASSERT(FALSE);
   }

   // *** Was there any error ***
   if( m_retcode == SQL_ERROR )
   {
      m_pODBCDatabase->DisplayODBCError( hstmt );
      ::SQLFreeStmt( hstmt, SQL_DROP );
      return FALSE;
   }

#ifdef _DEBUG
   if( m_retcode != SQL_SUCCESS && m_retcode != SQL_NEED_DATA )
   {
      m_pODBCDatabase->DisplayODBCError( hstmt );
   }
#endif

   // *** If we sent a pointer to statement assign it ***
   if( pHstmt != SQL_NULL_HSTMT )
   {
      *pHstmt = hstmt;
   }
   else 
   {
      // *** Drop the statement ***
      m_retcode = ::SQLFreeStmt( hstmt, SQL_DROP );
      if( m_retcode != SQL_SUCCESS )
      {
         m_pODBCDatabase->DisplayODBCError();
         return FALSE;
      }
   }

   return TRUE;
}


/*-------------------------------------------------------------------------------------------------
Name:	   DataAtExecution()
Type:    Function
Purpose: Complete a question that should have data at execution-time

Param:   hstmt = Statement that we are working on
         pDAE = pointer to a array of DATA_AT_EXECUTION-structures

Return:  BOOL, TRUE if everything went ok, otherwise FALSE

Se also:
(file)DECLARATIONS_FOR_ODBC.h
*/
BOOL CODBCRecordSet::DataAtExecution( HSTMT hstmt, PODBC_DATA_AT_EXECUTION pDAE )
{
   ASSERT( this );
   ASSERT( ::AfxIsValidAddress( pDAE, sizeof( ODBC_DATA_AT_EXECUTION ) ) );

   RETCODE retcode;         // Returncode from SQLParamData()
   int     iParameterIndex; // Parameter that want data
   UINT    uDAEIndex;       // Index on PDATA_AT_EXECUTION that has information on data for transfer

   if( hstmt == SQL_NULL_HSTMT ) // Should we use m_hstmt
   {
      ASSERT( m_hstmt != SQL_NULL_HSTMT );
      hstmt = m_hstmt;
   }

   do 
   {
      retcode = ::SQLParamData( hstmt, (PTR*)&iParameterIndex );
      if( retcode == SQL_NEED_DATA )
      {
         // *** Find the structure that has same index as requested ***
         uDAEIndex = 0;
         while( pDAE[uDAEIndex].m_sParameterIndex != (short)iParameterIndex )
         {
#ifdef _DEBUG
            // Warning if parameter is unrealistic
            if( (iParameterIndex < 1) || (iParameterIndex > 19) )
            {
               TRACE1("Has parameterindex been set correctly (%d)\n", iParameterIndex );
            }
#endif
            ASSERT( (UWORD)pDAE[uDAEIndex].m_sParameterIndex < 20 );
            ASSERT( uDAEIndex < 20 );
            uDAEIndex++;
         }

         // Transfer data to column in table 
         m_retcode = ::SQLPutData( hstmt, pDAE[uDAEIndex].m_pBuffer, pDAE[uDAEIndex].m_uBufferLength ); 
         if( m_retcode != SQL_SUCCESS )
         {
            m_pODBCDatabase->DisplayODBCError( hstmt );
            return FALSE;
         }
      }

#ifdef _DEBUG
      if( (retcode != SQL_NEED_DATA) && (retcode != SQL_SUCCESS) )
      {
         m_pODBCDatabase->DisplayODBCError( hstmt );
         return FALSE;
      }
#endif

   } while( retcode == SQL_NEED_DATA );

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Requery()
 Type:   Function
 Purpose:Hmta ett nytt resultat frn databasen
 
 Param:  pszSQL = pekare till den strng innehller SQL-frgan

 Info:
 Detta nya resultat som skall hmtas fr inte frndras i SELECT-delen. De kolumner som r bundna 
 mste ha en exakt motsvarighet fr de buffertar som r kopplade fr den tidigare frgan.
*/
BOOL CODBCRecordSet::Requery( LPCSTR pszSQL )
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );

   m_retcode = ::SQLFreeStmt(m_hstmt, SQL_CLOSE);
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }
   
   m_retcode = ::SQLPrepare(m_hstmt,(UCHAR*)pszSQL, SQL_NTS);
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }

   return Requery();
}

/*-------------------------------------------------------------------------------------------------
Name:	   Requery()
Type:    Function
Purpose: Get a new result but keep all bindings to buffers and so on.

Param:   hstmt = statement that should be required

Return:  BOOL, TRUE if everything went ok, otherwise FALSE
*/
BOOL CODBCRecordSet::Requery( HSTMT hstmt /*=SQL_NULL_HSTMT*/ )
{
   ASSERT_VALID( this );

   // Skickas ett statment med (d mste preparering av detta statment redan blivit gjord)
   if( hstmt )
   {
      m_retcode = ::SQLFreeStmt(m_hstmt, SQL_CLOSE);
      if( m_retcode != SQL_SUCCESS )
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         return FALSE;
      }

      m_hstmt = hstmt;
   }

   m_retcode = ::SQLExecute(m_hstmt);
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }

   MoveNext();

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:    MoveAbsolut()
Type:    Function
Purpose: Moves to the position that we request

Param:   lOffsetPos = the position that should become the active record.

Throw:   int, If anything goes wrong it throws an integer exeption
*/
void CODBCRecordSet::MoveAbsolut( long lOffsetPos )
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );
   ASSERT( lOffsetPos >= 0 );

   m_retcode = ::SQLExtendedFetch( m_hstmt, 
                                   SQL_FETCH_ABSOLUTE, 
                                   (SDWORD)lOffsetPos, 
                                   &dwDummy, 
                                   &m_wRowStatus );

   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode == SQL_NO_DATA_FOUND )
      {
         m_RecordPos = IsPastLastRecord;
      }
      else
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         throw ODBC_EXEPTION_MOVE_ABSOLUTE;
      }
   }
   else
   {
      m_RecordPos = IsOnValidRecord;
   }
}

/*-------------------------------------------------------------------------------------------------
Name:    MoveRelative()
Type:    Function
Purpose: Move to a position in a result that are relative to current position

Param:   lRelativePos = how man records that we should move

Throw:   int, If anything goes wrong it throws an integer exeption
*/
void CODBCRecordSet::MoveRelative( long lRelativePos )
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );

   m_retcode = ::SQLExtendedFetch( m_hstmt, 
                                   SQL_FETCH_RELATIVE, 
                                   (SDWORD)lRelativePos, 
                                   &dwDummy, 
                                   &m_wRowStatus );

   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode == SQL_NO_DATA_FOUND )
      {
         if( lRelativePos > 0 )
         {
            m_RecordPos = IsPastLastRecord;
         }
         else
         {
            m_RecordPos = IsBeforeFirstRecord;
         }
      }
      else
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         throw ODBC_EXEPTION_MOVE_RELATIVE;
      }
   }
   else
   {
      m_RecordPos = IsOnValidRecord;
   }
}

/*-------------------------------------------------------------------------------------------------
Name:    MoveToBookmark()
Type:    Function
Purpose: Flytta till ett bokmrke (radnummer)

Param:   lBookmark = vrde fr det bokmrke vi skall flytta till.

Throw:   int, If anything goes wrong it throws an integer exeption
*/
void CODBCRecordSet::MoveToBookmark( long lBookmark )
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );

   if( m_uFlags & ODBC_BOOKMARKS )
   {
      m_retcode = ::SQLExtendedFetch( m_hstmt, 
                                      SQL_FETCH_BOOKMARK, 
                                      (SDWORD)lBookmark, 
                                      &dwDummy, 
                                      &m_wRowStatus );
      if( m_retcode != SQL_SUCCESS )
      {
         if( m_retcode == SQL_NO_DATA_FOUND )
         {
            m_RecordPos = IsPastLastRecord;
         }
         else
         {
            m_pODBCDatabase->DisplayODBCError( m_hstmt );
            throw ODBC_EXEPTION_MOVE_BOOKMARK;
         }
      }
      else
      {
         m_RecordPos = IsOnValidRecord;
      }
   }
   else
   {
      MoveAbsolut( lBookmark );
   }
}


/*-------------------------------------------------------------------------------------------------
Name:    MoveNext()
Type:    Function
Purpose: Move to next record

Throw:   int, If anything goes wrong it throws an integer exeption
*/
void CODBCRecordSet::MoveNext()
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );

   m_retcode = ::SQLExtendedFetch( m_hstmt, SQL_FETCH_NEXT, 0, &dwDummy, &m_wRowStatus );

   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode == SQL_NO_DATA_FOUND )
      {
         m_RecordPos = IsPastLastRecord;
      }
      else
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         throw ODBC_EXEPTION_MOVE_NEXT;
      }
   }
   else
   {
      m_RecordPos = IsOnValidRecord;
   }
}

/*-------------------------------------------------------------------------------------------------
Name:    MovePrevious()
Type:    Function
Purpose: Flytta till fregende post

Throw:   int, If anything goes wrong it throws an integer exeption
*/
void CODBCRecordSet::MovePrevious()
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );

   m_retcode = ::SQLExtendedFetch( m_hstmt, SQL_FETCH_PRIOR, 0, &dwDummy, &m_wRowStatus );

   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode == SQL_NO_DATA_FOUND )
      {
         m_RecordPos = IsBeforeFirstRecord;
      }
      else
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         throw ODBC_EXEPTION_MOVE_PREVIOUS;
      }
   }
   else
   {
      m_RecordPos = IsOnValidRecord;
   }
}

/*-------------------------------------------------------------------------------------------------
Name:    MoveLast()
Type:    Function
Purpose: Med denna Function flyttas aktuell position till den posten

Throw:   int, If anything goes wrong it throws an integer exeption
*/
void CODBCRecordSet::MoveLast()
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );

   m_retcode = ::SQLExtendedFetch( m_hstmt, SQL_FETCH_LAST, 0, &dwDummy, &m_wRowStatus );

   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode == SQL_NO_DATA_FOUND )
      {
         m_RecordPos = IsBeforeFirstRecord;
      }
      else
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         throw ODBC_EXEPTION_MOVE_LAST;
      }
   }
   else
   {
      m_RecordPos = IsOnValidRecord;
   }
}

/*-------------------------------------------------------------------------------------------------
Name:    MoveFirst()
Type:    Function
Purpose: Move to first record in result

Throw:   int, If anything goes wrong it throws an integer exeption
*/
void CODBCRecordSet::MoveFirst()
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );

   m_retcode = ::SQLExtendedFetch( m_hstmt, SQL_FETCH_FIRST, 0, &dwDummy, &m_wRowStatus );

   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode == SQL_NO_DATA_FOUND )
      {
         m_RecordPos = IsBeforeFirstRecord;
      }
      else
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         throw ODBC_EXEPTION_MOVE_FIRST;
      }
   }
   else
   {
      m_RecordPos = IsOnValidRecord;
   }
}

/*-------------------------------------------------------------------------------------------------
Name:    Search()
Type:    Function
Purpose: Search for a value in a record

Param:   iColumn = column that value can be found at
         dwValue = value that we ar looking for
         bOnFailGoBack = if TRUE go back to current record when seeked value vas not fond

Return:  If value wasn't found FALSE, otherwise TRUE
*/
BOOL CODBCRecordSet::Search( int iColumn, DWORD dwValue, BOOL bOnFailGoBack /*=FALSE*/ )
{
   DWORD dwRecord = 0;  // Current record
   DWORD dwValueFound;  // Value in column

   if( m_RecordPos == IsOnValidRecord )
   {
      // GetBookmark( &dwRecord );
   }

   MoveFirst();                                  // Move to first pos

   while( m_RecordPos == IsOnValidRecord )       // Is cursor on valid record
   {
      GetDWORDInColumn( iColumn, dwValueFound ); // Get value

      if( dwValueFound == dwValue )              // Same as the value we are looking for 
      {
         return TRUE;                              
      }

      MoveNext();                                // Move to next pos
   }

   if( bOnFailGoBack )                           // Return to old pos
   {
      MoveAbsolut( dwRecord );
   }

   return FALSE;
}

/*-------------------------------------------------------------------------------------------------
Name:    Search()
Type:    Function
Purpose: Search for a value in a record

Param:   iColumn = column that value can be found at
         pszValue = value that we ar looking for
         bCase = is matching is case sensitive
         bExact = value should be matched exact
         bOnFailGoBack = if TRUE go back to current record when seeked value vas not fond

Return:  If value wasn't found FALSE, otherwise TRUE
*/
BOOL CODBCRecordSet::Search( int iColumn, LPCSTR pszValue, BOOL bOnFailGoBack )
{
   DWORD   dwRecord = 0;
   CString string;

   if( m_RecordPos == IsOnValidRecord )
   {
      GetBookmark( &dwRecord );
   }

   MoveFirst();                                  // Move to first pos

   while( m_RecordPos == IsOnValidRecord )       // Is cursor on valid record
   {
      GetValueInColumn( iColumn, string );       // Get value
      
      if( string == pszValue )                   // Same as the value we are looking for 
      {
         return TRUE;                              
      }

      MoveNext();                                // Move to next pos
   }

   if( bOnFailGoBack )                           // Return to old pos
   {
      MoveAbsolut( dwRecord );
   }

   return FALSE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Add()      
 Type:   Function
 Purpose:Lgg till en post i aktuell tabell som cursorn r kopplad till

 Return: BOOL = SANT om allt fungerat som det skall annars FALSKT
*/
BOOL CODBCRecordSet::Add()
{
   ASSERT( m_hstmt );
   ASSERT_VALID( this );
   
   m_retcode = ::SQLSetPos( m_hstmt, 1, SQL_ADD, SQL_LOCK_NO_CHANGE );
   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode == SQL_SUCCESS_WITH_INFO )
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
      }
      else
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         return FALSE;
      }// endif
   }// endif

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Update()
 Type:   Function
 Purpose:Uppdatera en rad som finns i det resultat som hlls av cursorn

 Param:  wRow = den rad som skall uppdateras, om 0 s uppdateras den raden som cursorn r p.

 Return: BOOL = SANT om allt gtt som det skall annars FALSKT
*/
BOOL CODBCRecordSet::Update( UWORD wRow /*=0*/ )
{
   ASSERT( m_hstmt );
   ASSERT_VALID( this );

   if( wRow == 0 )
   {
      wRow = (UWORD)GetRowNumber();
      if( wRow == 0 ) return FALSE;;
   }
   
   // *** Ls raden innan vi kan gra uppdateringen ***
   m_retcode = ::SQLSetPos(m_hstmt, wRow, SQL_POSITION, SQL_LOCK_EXCLUSIVE);
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }

   // *** Uppdatera raden ***
   m_retcode = ::SQLSetPos( m_hstmt, wRow, SQL_UPDATE, SQL_LOCK_NO_CHANGE );
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }// endif

   // *** Ls upp raden ***
   m_retcode = ::SQLSetPos(m_hstmt, wRow, SQL_POSITION, SQL_LOCK_UNLOCK);
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Delete()
 Type:   Function
 Purpose:Radera en post i resultatet
 
 Param:  wRow = den rad som skall uppdateras, om 0 s uppdateras den raden som cursorn r p.

 Return: BOOL = SANT om allt gtt som det skall annars FALSKT
*/
BOOL CODBCRecordSet::Delete( UWORD wRow /*=0*/ )
{
   ASSERT( m_hstmt );
   ASSERT_VALID( this );

   if( wRow == 0 )
   {
      // Hmta aktuellt radnummer
      wRow = (UWORD)GetRowNumber();
      if( wRow == 0 ) return FALSE;;
   }
   
   // Radera raden
   m_retcode = ::SQLSetPos( m_hstmt, wRow, SQL_DELETE, SQL_LOCK_NO_CHANGE );
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }// endif
   
   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:    IsFieldNull()
Type:    Function
Purpose: Check if a value in column is SQL_NULL_DATA (no data in field)

Param:   uColumn = index for the column that are going to be checked

Return:  TRUE if field is NULL otherwise FALSE.
*/
BOOL CODBCRecordSet::IsFieldNull( UINT uColumn )
{
   ASSERT_VALID( this );
   ASSERT(uColumn <= m_uFieldCount);
   ASSERT( m_RecordPos == IsOnValidRecord );
   ASSERT( m_ptrarrayFieldValue[uColumn] != NULL );

   return ((int)*(int*)m_ptrarrayFieldValue[uColumn] == SQL_NULL_DATA) ? TRUE : FALSE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   GetValueInColumn()
 Type:   Function
 Purpose:Hmta ett vrde frn en kolumn och lgg det i "string"

 Param:  uColumn = frn vilken kolumn som vrdet skall hmtas ifrn
         string = vrdet som hmtas lggs in i hr
*/
void CODBCRecordSet::GetValueInColumn( UINT uColumn, CString& string )
{
   ASSERT_VALID( this );
   ASSERT(uColumn <= m_uFieldCount);
   ASSERT( m_RecordPos == IsOnValidRecord );

   char pszBuffer[20];

#ifdef _DEBUG
   if( m_ptrarrayFieldValue[uColumn] == NULL )
   {
      TRACE1("Field (%d) is unbound program will probably crash in release\n", uColumn);
      string.Empty();
      return;
   }
#endif

   switch( LOWORD( m_dwarrayColumnType[uColumn] ) )
   {
   case SQL_CHAR :
   case SQL_VARCHAR :
      string = (char*)m_ptrarrayFieldValue[uColumn];
      string.TrimRight();
      break;
   case SQL_INTEGER :
      _itoa( (int)*(int*)m_ptrarrayFieldValue[uColumn], pszBuffer, 10 );
      string = pszBuffer;
      break;
   case SQL_DOUBLE :
      sprintf( pszBuffer, "%0.2f",(double)*(double*)m_ptrarrayFieldValue[uColumn] );
      string = pszBuffer;
      break;
   case SQL_SMALLINT :
      _itoa( (int)(short)*(short*)m_ptrarrayFieldValue[uColumn], pszBuffer, 10 );
      string = pszBuffer;
      break;
   case SQL_NUMERIC :
   case SQL_DECIMAL :
      string = static_cast<char*>(m_ptrarrayFieldValue[uColumn]);
      string.TrimRight();
      break;
   default:
      TRACE1("Unknown columntype %d\n", LOWORD( m_dwarrayColumnType[uColumn] ) );
      ASSERT(FALSE);
   }// endswitch

}

/*-------------------------------------------------------------------------------------------------
 Name:   GetValueInColumn()
 Type:   Function
 Purpose:Hmta ett vrde frn en kolumn och lgg det i "string"

 Param:  uColumn = frn vilken kolumn som vrdet skall hmtas ifrn
*/
void CODBCRecordSet::GetValueInColumn( UINT uColumn, void*& pValue )
{
   ASSERT_VALID( this );
   ASSERT(uColumn <= m_uFieldCount);
   ASSERT( m_RecordPos == IsOnValidRecord );

   pValue = m_ptrarrayFieldValue[uColumn];
}

/*-------------------------------------------------------------------------------------------------
 Name:   GetDWORDInColumn()
 Type:   Function
 Purpose:Hmta ett vrde frn en kolumn av type SQL_INTEGER

 Param:  uColumn = index p den kolumn som vi skall hmta ett vrde ifrn
         dwValue = i denna variabel kommer vrdet att lggas in
*/
void CODBCRecordSet::GetDWORDInColumn( UINT uColumn, DWORD& dwValue )
{
   ASSERT_VALID( this );
   ASSERT(uColumn <= m_uFieldCount);
   ASSERT( LOWORD( m_dwarrayColumnType[uColumn] ) == SQL_INTEGER );
   ASSERT( m_RecordPos == IsOnValidRecord );

   dwValue = (DWORD)*(int*)m_ptrarrayFieldValue[uColumn];
}

/*-------------------------------------------------------------------------------------------------
Name:    ConvertColumnsToText()
Type:    Function
Purpose: Convert selected values to a text with separators between columns

Param:   piColumn =    array with the columnindexes that are going to be added (if less than 0 stop)
         stringText =  the text made from columns
         chSeparator = sign that are placed between columns
*/
void CODBCRecordSet::ConvertColumnsToText( int* piColumn, CString& stringText, char chSeparator )
{
   char    pszText[500];     // Buffer for building text
   char*   pszColumn;        // Ponter to column
   UINT    uCounter = 0;     // Counter for counting chars
   UINT    uTextCounter = 0; // Counter for holding pos in "pszText"
   UINT    uColumn;
   CString string;           // String for getting values in columns

   while( piColumn[uCounter] >= 0 )
   {
      ASSERT( m_ptrarrayFieldValue[piColumn[uCounter]] );
      ASSERT( piColumn[uCounter] < (int)m_uFieldCount );
      ASSERT( uCounter < 30 );

      uColumn = piColumn[uCounter];

      // *** Check if column is text (optimisation) ***
      if( LOWORD( m_dwarrayColumnType[uColumn] ) == SQL_CHAR || 
          LOWORD( m_dwarrayColumnType[uColumn] ) == SQL_VARCHAR )
      {
         // *** Copy all characters in buffer ***
         pszColumn = static_cast<char*>(m_ptrarrayFieldValue[uColumn]);
         while( *pszColumn )
         {
            pszText[uTextCounter] = *pszColumn;
            uTextCounter++;
            pszColumn++;
         }
         // *** Remove invisible chars ***
         uTextCounter--;
         while( pszText[uTextCounter] <= ' ' ) uTextCounter--;
      }
      else
      {
          GetValueInColumn( uColumn, string );
          strcpy( &pszText[uTextCounter], string );
          uTextCounter += string.GetLength() + 1;
      }

      pszText[uTextCounter] = chSeparator;
      uTextCounter++;
      uCounter++;
      ASSERT( uTextCounter < 500 );
   }// end while
   
   pszText[uTextCounter-1] = '\0';

   stringText = pszText;
}

/*-------------------------------------------------------------------------------------------------
Name:	   GetData()
Type:    Funktion
Purpose: Get data from column            

Param:   wColumn = which column data should be fetched from.
         pVoid = pointer to storage where data should be transfered to.
         iMaxBytes = max byte-count that should be retrieved from column.
         pdwBytesInColumn = Total number of bytes available in "pVoid"

Return:  int, bytes that actually was fetched from column, -1 on error.
*/
RETCODE CODBCRecordSet::GetData( UWORD wColumn, void* pVoid, int iMaxBytes, DWORD* pdwBytesInColumn )
{
   m_retcode = ::SQLGetData( m_hstmt, 
                             wColumn+1, 
                             static_cast<SWORD>(m_dwarrayColumnType[wColumn]), 
                             static_cast<PTR>(pVoid),
                             static_cast<SDWORD>(iMaxBytes),
                             (SDWORD*)pdwBytesInColumn );
   
   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode != SQL_SUCCESS_WITH_INFO && m_retcode != SQL_NO_DATA_FOUND )
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         return -1;
      }
   }

   return m_retcode;
}

/*-------------------------------------------------------------------------------------------------
Name:	   GetData()
Type:    Funktion
Purpose: Get data from column            

Param:   wColumn = which column data should be fetched from.
         pVoid = pointer to storage where data should be transfered to.
         iMaxBytes = max byte-count that should be retrieved from column.
         pdwBytesInColumn = Total number of bytes available in "pVoid"

Return:  int, bytes that actually was fetched from column, -1 on error.
*/
RETCODE CODBCRecordSet::GetData( PODBC_GET_DATA pget_data )
{
   m_retcode = ::SQLGetData( m_hstmt, 
                             pget_data->wColumn + 1, 
                             pget_data->wCType, 
                             pget_data->pBuffer,
                             pget_data->dwMaxBytes,
                             &pget_data->dwByteRead );
   
   if( m_retcode != SQL_SUCCESS )
   {
      if( m_retcode != SQL_SUCCESS_WITH_INFO && m_retcode != SQL_NO_DATA_FOUND )
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         return -1;
      }
   }

   return m_retcode;
}

/*-------------------------------------------------------------------------------------------------
 Name:   GetBookmark()
 Type:   Function
 Purpose:Fetch a bookmark

 Param:  plBookmark = pointer to a place where bookmark should be placed
*/
BOOL CODBCRecordSet::GetBookmark( ULONG* plBookmark )
{
   ASSERT_VALID( this );
   ASSERT( m_hstmt );
   

   UWORD wOption;    // Den typ av bokmrke som vi skall hmta

   if( m_RecordPos != IsOnValidRecord )
   {
      return FALSE;
   }

   // *** Kolla vilken typ av cursor som vi har skapat och utifrn detta kan vi bestmma vilken typ av 
   // bokmrke som skall hmtas ***
   if( m_uFlags & ODBC_BOOKMARKS )
   {
      wOption = SQL_FETCH_BOOKMARK;
   }
   else
   {
      wOption = SQL_ROW_NUMBER;
   }

   // Hmta bokmrke
   m_retcode = ::SQLGetStmtOption( m_hstmt, wOption, static_cast<PTR>(plBookmark) ); 
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return FALSE;
   }

   return TRUE;
}


/*-------------------------------------------------------------------------------------------------
 Name:   BindColumns()
 Type:   Function                 
 Purpose:Med denna Function skall vi binda samman kolumner i ett resultat till de skapade 
         buffertarna. 
*/
BOOL CODBCRecordSet::BindColumns()
{
   ASSERT( m_hstmt );
   ASSERT_VALID( this );

   UINT     uCounter = 0;    // Rknare fr att man skall kunna lopa igenom alla flt som binds upp
   SDWORD   dwBufferSize;    // Storleken p den buffer som binds till ett resultat
   SWORD    wColumnType;     // Vilken typ av kolumn som det r vi skall binda upp

   // *** Lopa igenom alla kolumner som r kopplade till resultatet fr att koppla dem till en 
   // buffert. ***
   while( uCounter < m_uFieldCount )
   {
      wColumnType = LOWORD( m_dwarrayColumnType[uCounter] );             // Kolumntypen
      if( wColumnType == SQL_CHAR || wColumnType == SQL_VARCHAR )        // r det text ?
      {
         dwBufferSize = (SDWORD)HIWORD( m_dwarrayColumnType[uCounter] ); // Lngden p textfltet
      }
      else if( m_ptrarrayFieldValue[uCounter] != NULL )
      {
         dwBufferSize = (SDWORD)module_pdwBufferSize[LOWORD(m_dwarrayColumnType[uCounter])];
      }
      else
      {
         dwBufferSize = 0;
      }

      // *** Kontrollera att vi skapat upp en buffer och i s fall koppla samman med kolumnen ***                         
      if( dwBufferSize != 0 )
      {
         m_retcode = ::SQLBindCol( m_hstmt, 
                                   static_cast<UWORD>(uCounter+1),
                                   wColumnType,
                                   m_ptrarrayFieldValue[uCounter],
                                   dwBufferSize,
                                   &m_pdwStoredBytesInBuffer[uCounter] );
      }

      if( m_retcode != SQL_SUCCESS )
      {
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         return FALSE;
      }

      uCounter++;
   }
   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
Name:	   BindParameter()
Type:    Function
Purpose: Bind a parameter in SQL-question

Param:   iCount = Number of pointer in pBP-array
         pBP = Pointers to tagBIND_PARAMETER-structure

Return: BOOL, TRUE if everything went ok, otherwise FALSE

Se also:
(file)DECLARATIONS_FOR_ODBC.h
*/
BOOL CODBCRecordSet::BindParameter( int iCount, PODBC_BIND_PARAMETER pBP )
{
   HSTMT             hstmt;   // Handle to statement
   PODBC_BIND_PARAMETER   pbp;// Point to structure that contains information about parameter

   for( int iParameterIndex = 0; iParameterIndex < iCount; iParameterIndex++ )
   {
      pbp = &pBP[iParameterIndex];     // Get pointer to structure for information on how to bind parameter

      if( pbp->m_hstmt == SQL_NULL_HSTMT )
      {
         hstmt = m_hstmt; 
      }
   
      ASSERT( hstmt != SQL_NULL_HSTMT );
      m_retcode = ::SQLBindParameter(  hstmt, 
                                       pbp->m_wIndex,            
                                       pbp->m_wParamType, 
                                       pbp->m_wCType, 
                                       pbp->m_wSQLType, 
                                       pbp->m_dwPrecision, 
                                       pbp->m_wScale, 
                                       pbp->m_pBuffer, 
                                       pbp->m_dwMaxBytes, 
                                       pbp->m_pdwBytesRetrieved );
      
      if( m_retcode != SQL_SUCCESS )
      {
         TRACE1("Information for parameter (%d) was not correct\n",pbp->m_wIndex);
         m_pODBCDatabase->DisplayODBCError( m_hstmt );
         return FALSE;
      }
   }

   return TRUE;
}

/*-------------------------------------------------------------------------------------------------
 Name:   Close()
 Type:   Function
 Purpose:Stng igen ett resultat
*/
void CODBCRecordSet::Close()
{
   if( m_hstmt != SQL_NULL_HSTMT )
   {
      m_retcode = ::SQLFreeStmt( m_hstmt, SQL_DROP );
      m_hstmt = SQL_NULL_HSTMT;
      m_RecordPos = Closed;
      m_uFlags = 0;
   }

   DeleteBuffers();
}

/*-------------------------------------------------------------------------------------------------
 Name:   DeleteBuffers()
 Type:   Function
 Purpose:Raderar de buffertar som r allokerade fr att hlla mellanlagra vrden frn ett resultat
*/
void CODBCRecordSet::DeleteBuffers()
{
   while( m_uFieldCount )
   {
      m_uFieldCount--;
      delete m_ptrarrayFieldValue[m_uFieldCount];
   }
   m_ptrarrayFieldValue.RemoveAll();
}


/*-------------------------------------------------------------------------------------------------
Name:    GetRowNumber()
Type:    Function
Purpose: Get the rownumber that cursor is on

Return:  DWORD, rownumber that cursor is on.
*/
DWORD CODBCRecordSet::GetRowNumber()
{
   DWORD dwRowNumber;

   m_retcode = ::SQLGetStmtOption( m_hstmt, SQL_ROW_NUMBER, (PTR)&dwRowNumber ); 
   if( m_retcode != SQL_SUCCESS )
   {
      m_pODBCDatabase->DisplayODBCError( m_hstmt );
      return 0;
   }

   return dwRowNumber;
}

/************************************************
   *   Debug
   *
 * * *
  ***
   *
*/

#ifdef _DEBUG
void CODBCRecordSet::AssertValid() const
{
	CObject::AssertValid();
   ASSERT(m_pODBCDatabase->IsKindOf( RUNTIME_CLASS(CODBCDatabase) ) );
}

void CODBCRecordSet::Dump(CDumpContext& dc) const
{
	CObject::Dump(dc);
   dc << "m_retcode = " << (int)m_retcode << "\n" << "m_RecordPos = " << (int)m_RecordPos << '\n';
   dc << "m_uFlags = " << m_uFlags << '\n' << "m_hstmt = " << m_hstmt << '\n';
}
#endif //_DEBUG








