/*
   FILNAMN:    ODBCRecordSet.h

   DELSYSTEM:  

   SYFTE:      Kunna kontrollera statements via ODBC

   SKAPAD:     951004 Per Ghosh
   MILJ:      Visual C++


INFO:


*/


#ifndef __ODBCRECORDSET_H_
#define __ODBCRECORDSET_H_

class CODBCDatabase;

class CODBCRecordSet : public CObject
{
   friend class CODBCDatabase;

   DECLARE_DYNAMIC(CODBCRecordSet)

// Construction/Destruction
public:
	CODBCRecordSet( CODBCDatabase* pODBCDatabase );          
	virtual ~CODBCRecordSet();

   enum RecordPos { IsBeforeFirstRecord, IsPastLastRecord, IsOnValidRecord, NoRows, Closed, Empty };

// Attributes
public:
   HSTMT          m_hstmt;              // Handtag till resultatet

   CODBCDatabase* m_pODBCDatabase;      // Pekare till databasen
   UINT           m_uFieldCount;        // Antal flt som r kopplade till klassen
	UINT           m_uFlags;             // Information om det aktiva resultatet

   UWORD          m_wRowStatus;         // Har statusen fr ett hmtad rad (ej statiska cursors)
   RecordPos      m_RecordPos;          // Information om vad cursorn befinner sig m.m.
   RETCODE        m_retcode;            // Returvrde vid anrop av ODBC-API.

   CStringArray   m_sarrayColumnName;   // Namnen p de kolumner som r kopplade
   CPtrArray      m_ptrarrayFieldValue; // Pekare till den buffer som skall mellanlagra vrdena
   CDWordArray    m_dwarrayColumnType;  // Information om vad det r fr typ p kolumnen
   SDWORD         m_pdwStoredBytesInBuffer[MAX_FIELDS];// lagrar det antal bytes som lsts frn tabell

// Get/Set
public:
   UINT  GetFieldCount() { return m_uFieldCount; }
   HSTMT GetHSTMT() { return m_hstmt; }
   UWORD GetRowStatus() { return m_wRowStatus; }
   int   GetRETCODE() { return m_retcode; }
   
   UINT  GetFlags() { return m_uFlags; }
   void  SetFlags( UINT uFlags ) { m_uFlags |= uFlags; }
   void  ClearFlags( UINT uFlags ) { m_uFlags &= ~(uFlags); } 


// Operations
public:
	BOOL AllocStmt();                             // Allocate memory for statement

   // *** execute question and attach a cursor ***
   BOOL Open( LPCSTR pszSQL );
	BOOL Open( LPCSTR pszSQL, WORD wOptionCount, PODBC_OPTION pOptions );
	BOOL Open( LPCSTR pszSQL, UINT uCursorType, BOOL bBind = TRUE, BOOL bInitialize = FALSE );

   BOOL Prepare( LPCSTR pszSQL );                // Prepare statement
	
   BOOL SetStmtOption( WORD wOption, UINT uParam );
	BOOL SetStmtOption( UINT uOptionCount, PODBC_OPTION pOptions );

   BOOL Initialize( PODBC_QUESTION_BUFFERINFO pQB );  // Create buffers for a result
   BOOL Initialize( const DWORD* pdwInfo );      // Create buffers for a result
   BOOL Initialize( const CDWordArray& dwarray );// Create buffers for a result
	BOOL Initialize();
	
	BOOL Execute( HSTMT hstmt = SQL_NULL_HSTMT ); // execute statement
	BOOL Execute( LPCSTR pszSQL, HSTMT* pHstmt = SQL_NULL_HSTMT, UINT uOptions = ODBC_EXEC_DIRECT );
	BOOL DataAtExecution( HSTMT hstmt, PODBC_DATA_AT_EXECUTION pDAE );

   BOOL Requery( LPCSTR pszSQL );                // Get a new result 
	BOOL Requery( HSTMT hstmt = SQL_NULL_HSTMT);  // Get a new result 

   inline BOOL IsOnRecord();                     // Check to se if cursor is on record
   inline BOOL IsBOF();
   inline BOOL IsEOF();
   inline BOOL HasRecords();

   void MoveAbsolut( long lOffsetPos );          // Flytta till den absoluta posten
   void MoveRelative( long lRelativePos );       // Flytta till den relativa posten
   void MoveToBookmark( long lBookmark );        // Flytta till ett bokmrke
   void MoveNext();                              // Flytta till nsta post     
   void MovePrevious();                          // Flytta till fregende post
   void MoveLast();                              // Flytta till den sista posten
   void MoveFirst();                             // Flytta till den frsta posten

	BOOL Search( int iColumn, DWORD dwValue, BOOL bOnFailGoBack = FALSE );
	BOOL Search( int iColumn, LPCSTR pszValue, BOOL bOnFailGoBack );

   BOOL Add();                                   // Lgg till en post till tabellen
	BOOL Update( UWORD wRow = 0 );                // Uppdatera en post
	BOOL Delete( UWORD wRow = 0 );                // Radera en post

	BOOL IsFieldNull( UINT uColumn );             // Check if field has data

   void GetDWORDInColumn( UINT uColumn, DWORD& dwValue ); // Hmta ett vrde frn en kolumn
	void GetValueInColumn( UINT uColumn, void*& pValue );  // Stt pekare till buffer fr kolulmn
	void GetValueInColumn( UINT uColumn, CString& string );// Hmta ett vrde frn en kolumn
	void ConvertColumnsToText( int* piColumn, CString& stringRecord, char chSeparator );

   RETCODE GetData( PODBC_GET_DATA pget_data );  // Gets data from a column in a result
 	RETCODE GetData( UWORD wColumn, void* pVoid, int iMaxBytes, DWORD* pdwBytesInColumn );

   inline void UseBookmark();                    // Stt cursortypen s att bokmrke anvnds
   BOOL GetBookmark( ULONG* plBookmark );        // Hmta ett bokmrke (radnummer)
   BOOL GetRowIndex( ULONG* plBookmark ) { return GetBookmark( plBookmark ); }
   
   
   BOOL BindColumns();                           // Attach columns to allocated buffers.
	BOOL BindParameter( int iCount, PODBC_BIND_PARAMETER pBP );// Bind parameters in a qustion
	void Close();                                 // Close statement
   
// Overrides  

// Implementation 
public:
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

private:
 	void DeleteBuffers();
	DWORD GetRowNumber();

};

inline void CODBCRecordSet::UseBookmark()
{
   m_uFlags |= ODBC_BOOKMARKS;
   SetStmtOption( SQL_USE_BOOKMARKS, SQL_UB_ON );
}

inline BOOL CODBCRecordSet::IsOnRecord()
{
   return (m_RecordPos == IsOnValidRecord) ? TRUE : FALSE;
}

inline BOOL CODBCRecordSet::IsBOF()
{
   return (m_RecordPos == IsBeforeFirstRecord) ? TRUE : FALSE;
}

inline BOOL CODBCRecordSet::IsEOF()
{
   return (m_RecordPos == IsPastLastRecord) ? TRUE : FALSE;
}

inline BOOL CODBCRecordSet::HasRecords()
{
   ASSERT( m_hstmt != SQL_NULL_HSTMT );
   return (m_uFlags & ODBC_NO_RECORDS) ? FALSE : TRUE;
}

#endif

/*
QUESTIONS:
Q: How do I transfer BINARY-columndata ?
A: When data of binarytype is going to be transferred you have to use parameter's. First you 
   prepare(::SQLPrepare()) the question then bind(::SQLBindParameter())the parameter's in 
   question. When binding parameter of binary-type set the pcbValue (transferred bytes) to 
   SQL_LEN_DATA_AT_EXEC(length) and rgbValue (pointer to buffer) to the buffer that contains data
   or index for the parameter.
   Execute(::SQLExecute()) the question. If the returnvalue is SQL_NEED_DATA you have to find out
   which parameter that data is refering to. Use ::SQLParamData() to retrive information about 
   which it is. Then use ::SQLPutData() or ::SQLGetData() for transmitting data.
   When ::SQLParamData() doesn't return SQL_NEED_DATA the question is ready.
L: ODBC 2.x Programmer's Reference and SDK Guide, INSIDE ODBC

Q:
A: 
   
*/ 